//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stdio.h>
#include <pgapplication.h>
#include "ArmyDialog.h"
#include "ItemDialog.h"
#include "maptile.h"
#include "defs.h"
#include "hero.h"

ArmyDialog::ArmyDialog(Army* army, PG_Widget* parent, PG_Rect rect)
    :PG_Window(parent, rect, _("Army info"),  PG_Window::MODAL), d_army(army)
{
    char buffer[61];
    buffer[60] = '\0';

    //I assume a size of 500*440 pixels here
    
    d_b_ok = new PG_Button(this, PG_Rect(350, 400, 130, 30),
                            _("OK"),1);
    d_b_ok->sigClick.connect(slot(*this, &ArmyDialog::b_ok_clicked));

    //the army pic
    d_b_pic = new PG_Button(this, PG_Rect(100, 100, 50, 50), "",2);
    d_b_pic->EnableReceiver(false);
    d_b_pic->SetIcon(d_army->getPixmap(), 0, 0);

    // If the army is a hero, add a button to have the user handle the items
    d_b_item = new PG_Button(this, PG_Rect(85, 185, 80, 25), _("Items"),3);
    d_b_item->sigClick.connect(slot(*this, &ArmyDialog::b_itemClicked));
    if (!d_army->isHero())
        d_b_item->Hide();
    
    //the name of the army
    d_l_name = new PG_Label(this, PG_Rect(250, 50, 200, 20), 
                            d_army->getName().c_str());
    if (d_army->isHero())
        d_l_name->SetText(dynamic_cast<Hero*>(d_army)->getName().c_str());
                            
    //the head of the table for the single values
    d_l_basic = new PG_Label(this, PG_Rect(400, 70, 45, 20), _("basic"));
    d_l_real = new PG_Label(this, PG_Rect(450, 70, 45, 20), _("real"));

    //and the values themselves
    new PG_Label(this, PG_Rect(250, 90, 95, 20), _("strength"));
    d_l_strengthb = new PG_Label(this, PG_Rect(400, 90, 45, 20), "");
    d_l_strengthr = new PG_Label(this, PG_Rect(450, 90, 45, 20), "");

    new PG_Label(this, PG_Rect(250, 110, 95, 20), _("ranged"));
    d_l_rangedb = new PG_Label(this, PG_Rect(400, 110, 45, 20), "");
    d_l_rangedr = new PG_Label(this, PG_Rect(450, 110, 45, 20), "");
    
    d_l_shots = new PG_Label(this, PG_Rect(400, 130, 100, 20), "");

    new PG_Label(this, PG_Rect(250, 150, 95, 20), _("defense"));
    d_l_defenseb = new PG_Label(this, PG_Rect(400, 150, 45, 20), "");
    d_l_defenser = new PG_Label(this, PG_Rect(450, 150, 45, 20), "");

    new PG_Label(this, PG_Rect(250, 170, 96, 20), _("vitality"));
    d_l_vitalityb = new PG_Label(this, PG_Rect(400, 170, 45, 20), "");
    d_l_vitalityr = new PG_Label(this, PG_Rect(450, 170, 45, 20), "");

    new PG_Label(this, PG_Rect(250, 190, 95, 20), _("hp"));
    d_l_hpb = new PG_Label(this, PG_Rect(400, 190, 45, 20), "");
    d_l_hpr = new PG_Label(this, PG_Rect(450, 190, 45, 20), "");

    new PG_Label(this, PG_Rect(250, 210, 95, 20), _("moves"));
    d_l_movesb = new PG_Label(this, PG_Rect(400, 210, 45, 20), "");
    d_l_movesr = new PG_Label(this, PG_Rect(450, 210, 45, 20), "");

    snprintf(buffer, 60, _("xp: %.2f"), d_army->getXP());
    d_l_xp = new PG_Label(this, PG_Rect(250, 240, 95, 20), buffer);
    snprintf(buffer, 60, _("level: %i"), d_army->getLevel());
    d_l_level = new PG_Label(this, PG_Rect(400, 240, 95, 20), buffer);

    if (d_army->getProduction()==0) 
         snprintf(buffer, 60, _("prod. time: None"));
    else 
         snprintf(buffer, 60,
                 ngettext("prod. time: %i turn","prod. time: %i turns",d_army->getProduction()),
                 d_army->getProduction());
    d_l_prod = new PG_Label(this, PG_Rect(350, 310, 145, 20), buffer);

    if (d_army->getProductionCost()==0) 
         snprintf(buffer, 60, _("prod. cost: None"));
    else 
         snprintf(buffer, 60, _("prod. cost: %i gold"), d_army->getProductionCost());
    d_l_prodcost = new PG_Label(this, PG_Rect(350, 330, 165, 20), buffer);

    snprintf(buffer, 60, _("Upkeep: %i gold"), d_army->getUpkeep());
    d_l_upkeep = new PG_Label(this, PG_Rect(350, 350, 145, 20), buffer);

    d_l_movebonusn = new PG_Label(this, PG_Rect(200, 270, 160, 20), _("move bonus"));
    for (int i = 0; i < 4; i++)
        d_l_movebonusv[i] = new PG_Label(this, PG_Rect(200, 300 + i*30, 100, 20), "");

    d_l_armybonusn = new PG_Label(this, PG_Rect(30, 270, 100, 20), _("bonus"));
    for (int i = 0; i < 6; i++)
        d_l_armybonusv[i] = new PG_Label(this, PG_Rect(30, 300 + i*20, 150, 20), "");


    // And finally, fill in the data
    fillData();
}

ArmyDialog::~ArmyDialog()
{
    delete d_b_ok;
    delete d_b_pic;
    delete d_b_item;
    delete d_l_name;
    delete d_l_basic;
    delete d_l_real;
    delete d_l_strengthb;
    delete d_l_strengthr;
    delete d_l_defenseb;
    delete d_l_defenser;
    delete d_l_vitalityb;
    delete d_l_vitalityr;
    delete d_l_hpb;
    delete d_l_hpr;
    delete d_l_movesb;
    delete d_l_movesr;
    delete d_l_prod;
    delete d_l_prodcost;
    delete d_l_upkeep;
    delete d_l_xp;
    delete d_l_level;
    delete d_l_movebonusn;
    delete d_l_armybonusn;

    for (int i = 0; i < 4; i++)
        delete d_l_movebonusv[i];

	for (int i = 0; i < 6; i++)
        delete d_l_armybonusv[i];
}

bool ArmyDialog::b_ok_clicked(PG_Button* btn)
{
    QuitModal();
    return true;
}

bool ArmyDialog::b_itemClicked(PG_Button* btn)
{
    int xpos = PG_Application::GetScreenWidth() / 2 - 200;
    int ypos = PG_Application::GetScreenHeight()/2 - 200;

    ItemDialog dialog(static_cast<Hero*>(d_army),
                      0, PG_Rect(xpos, ypos, 450, 380));

    dialog.Show();
    dialog.RunModal();
    dialog.Hide();

    fillData();

    return true;
}

void ArmyDialog::fillData()
{
    char buffer[61];
    buffer[60] = '\0';
    unsigned int bonus;

    PG_Application::SetBulkMode(true);
    
    // Update the info about the strength...
    snprintf(buffer, 60, "%i", d_army->getStat(Army::STRENGTH, false));
    d_l_strengthb->SetText(buffer);
    snprintf(buffer, 60, "%i", d_army->getStat(Army::STRENGTH));
    d_l_strengthr->SetText(buffer);
    if (d_army->getStat(Army::STRENGTH, false) < d_army->getStat(Army::STRENGTH))
        d_l_strengthr->SetFontColor(PG_Color(50, 255, 50));   // green highlight
    if (d_army->getStat(Army::STRENGTH, false) > d_army->getStat(Army::STRENGTH))
        d_l_strengthr->SetFontColor(PG_Color(200, 30, 30));   // red highlight

    // ... the ranged attack ...
    snprintf(buffer, 60, "%i", d_army->getStat(Army::RANGED, false));
    d_l_rangedb->SetText(buffer);
    snprintf(buffer, 60, "%i", d_army->getStat(Army::RANGED));
    d_l_rangedr->SetText(buffer);
    if (d_army->getStat(Army::RANGED) < d_army->getStat(Army::RANGED))
        d_l_rangedr->SetFontColor(PG_Color(50, 255, 50));
    if (d_army->getStat(Army::RANGED) > d_army->getStat(Army::RANGED))
        d_l_rangedr->SetFontColor(PG_Color(200, 30, 30));

    // ... the number of shots ...
    if (d_army->getStat(Army::SHOTS) == 0)
        snprintf(buffer, 60, _("no shots"));
    else
        snprintf(buffer, 60,
                ngettext("%i shot", "%i shots", d_army->getStat(Army::SHOTS)),
                d_army->getStat(Army::SHOTS));
    d_l_shots->SetText(buffer);
    
    // ... the defense ...
    snprintf(buffer, 60, "%i", d_army->getStat(Army::DEFENSE, false));
    d_l_defenseb->SetText(buffer);
    snprintf(buffer, 60, "%i", d_army->getStat(Army::DEFENSE));
    d_l_defenser->SetText(buffer);
    if (d_army->getStat(Army::DEFENSE, false) < d_army->getStat(Army::DEFENSE))
        d_l_defenser->SetFontColor(PG_Color(50, 255, 50));   //green highlight
    if (d_army->getStat(Army::DEFENSE, false) > d_army->getStat(Army::DEFENSE))
        d_l_defenser->SetFontColor(PG_Color(200, 30, 30));   //red highlight

    // ... the vitality ...
    snprintf(buffer, 60, "%i", d_army->getStat(Army::VITALITY, false));
    d_l_vitalityb->SetText(buffer);
    snprintf(buffer, 60, "%i", d_army->getStat(Army::VITALITY));
    d_l_vitalityr->SetText(buffer);
    if (d_army->getStat(Army::VITALITY, false) < d_army->getStat(Army::VITALITY))
        d_l_vitalityr->SetFontColor(PG_Color(50, 255, 50));   //green highlight
    if (d_army->getStat(Army::VITALITY, false) > d_army->getStat(Army::VITALITY))
        d_l_vitalityr->SetFontColor(PG_Color(200, 30, 30));   //red highlight

    // ... hitpoints ...
    snprintf(buffer, 60, "%i", d_army->getStat(Army::HP));
    d_l_hpb->SetText(buffer);
    snprintf(buffer, 60, "%i", d_army->getHP());
    d_l_hpr->SetText(buffer);
    if (d_army->getHP() < d_army->getStat(Army::HP))
        d_l_hpr->SetFontColor(PG_Color(200, 30, 30));

    // ... movement points ...
    snprintf(buffer, 60, "%i", d_army->getStat(Army::MOVES));
    d_l_movesb->SetText(buffer);
    snprintf(buffer, 60, "%i", d_army->getMoves());
    d_l_movesr->SetText(buffer);
    if (d_army->getMoves() < d_army->getStat(Army::MOVES))
        d_l_movesr->SetFontColor(PG_Color(200, 30, 20));

    // ... move bonus ...
    bonus = d_army->getStat(Army::MOVE_BONUS);

    for (int i = 0; i < 4; i++)
        d_l_movebonusv[i]->SetText("");
    
    if (bonus & Maptile::MAPTILE_WATER)
        d_l_movebonusv[0]->SetText(_("Water"));
    if (bonus & Maptile::MAPTILE_FOREST)
        d_l_movebonusv[1]->SetText(_("Forest"));
    if (bonus & Maptile::MAPTILE_HILL)
        d_l_movebonusv[2]->SetText(_("Hill"));
    if (bonus & Maptile::MAPTILE_SWAMP)
        d_l_movebonusv[3]->SetText(_("Swamp"));

    // ... and finally: the army bonus
    for (int i = 0; i < 6; i++)
        d_l_armybonusv[i]->SetText("");
    bonus = d_army->getStat(Army::ARMY_BONUS);
    if (bonus & Army::SHIP)
        d_l_armybonusv[0]->SetText(_("Move on water"));
    if (bonus & Army::LEADER)
        d_l_armybonusv[1]->SetText(_("Leadership"));
    if (bonus & Army::CAVALRY)
        d_l_armybonusv[2]->SetText(_("Cavalry"));
    if (bonus & Army::ANTICAVALRY)
        d_l_armybonusv[3]->SetText(_("Stronger vs cavalry"));
    if (bonus & Army::REGENERATE)
        d_l_armybonusv[4]->SetText(_("Regenerates"));
    if (bonus & Army::CRITICAL)
        d_l_armybonusv[5]->SetText(_("Assassin"));


    
    PG_Application::SetBulkMode(false);
    Update();
}

bool ArmyDialog::eventKeyDown(const SDL_KeyboardEvent* key)
{
    switch(key->keysym.sym)
    {
        case SDLK_RETURN:
            b_ok_clicked(0);
            break;
        case SDLK_i:
            b_itemClicked(0);
            break;
        default:
            break;
    }
    return true;
}
