//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef ARMYDIALOG_H
#define ARMYDIALOG_H

#include <pgwindow.h>
#include <pgbutton.h>
#include <pglabel.h>
#include "army.h"

class PG_Button;
class PG_Label;
class Army;

/** \brief The class for displaying details of an army
  * 
  * An instance of this class is created and shown when you right-click
  * on an army icon at the bottom of the game screen. It features the
  * picture and details about the stats of this army. The class itself
  * is very simple. The whole initialisation is to be found in the
  * constructor, the rest of the functionality is inherited from the
  * PG_Window class.
  */

class ArmyDialog : public PG_Window
{
    public:
        /** \brief The constructor
         * This is where the setup and arrangement of the data takes place
         */
        ArmyDialog(Army* army, PG_Widget* parent, PG_Rect rect);

        /** \brief The destructor */
        ~ArmyDialog();

        /** \brief The function to be called when you press the OK button*/
        bool b_ok_clicked(PG_Button* btn);

        /** \brief The function to be called when you press the "Items" button
          * below the army picture in case it is a hero.
          */
        bool b_itemClicked(PG_Button* btn);
        
    private:
        //! Fills the labels with dynamic information about the displayed unit
        void fillData();

        bool eventKeyDown(const SDL_KeyboardEvent* key);
        
        /** \brief several data, only to be used for displaying purposes*/
        Army* d_army;
        PG_Button* d_b_ok;
        PG_Button* d_b_pic;
        PG_Button* d_b_item;
        PG_Label* d_l_name;
        PG_Label* d_l_basic;
        PG_Label* d_l_real;
        PG_Label *d_l_strengthb, *d_l_strengthr;
        PG_Label *d_l_rangedb, *d_l_rangedr;
        PG_Label *d_l_shots;
        PG_Label *d_l_defenseb, *d_l_defenser;
        PG_Label *d_l_vitalityb, *d_l_vitalityr;
        PG_Label *d_l_hpb, *d_l_hpr;
        PG_Label *d_l_movesb, *d_l_movesr;
        PG_Label* d_l_prod;
        PG_Label* d_l_prodcost;
        PG_Label* d_l_upkeep;
        PG_Label* d_l_xp;
        PG_Label* d_l_level;
        PG_Label* d_l_movebonusn, *d_l_movebonusv[4];
        PG_Label* d_l_armybonusn, *d_l_armybonusv[6]; 
};

#endif
