//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "ArmyInfo.h"
#include <assert.h>
#include <pglabel.h>
#include "army.h"
#include "ArmyDialog.h"

//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<std::flush;}
#define debug(x)

ArmyInfo::ArmyInfo(Army* army, bool pressable, PG_Widget* parent, PG_Rect rect)
	:PG_ThemeWidget(parent, rect, true), d_army(army), d_pressed(false),
        d_pressable(pressable)
{
	d_pressed = d_army->isGrouped();

	// show number of moves
	char buffer[80];
	sprintf(buffer, "%i", d_army->getMoves());
	d_l_moves = new PG_Label(this, PG_Rect(13, 43, 20, 20), buffer);

	Show();
}

ArmyInfo::~ArmyInfo()
{
	delete d_l_moves;
}

void ArmyInfo::eventDraw(SDL_Surface* surface, const PG_Rect& rect)
{
	debug("eventDraw()");
	PG_ThemeWidget::eventDraw(surface, rect);

	assert(d_army);

	DrawBorder(rect, 1, !d_pressed);

	// show pic of army in the button
	PG_Rect r(2, 2, 40, 40);
	SDL_BlitSurface(d_army->getPixmap(), NULL, surface, &r);

	// draw the hitpoints-bar
	SDL_Color red = {255, 0, 0};
	SDL_Color green = {0, 255, 0};

    float factor = (float)d_army->getHP() / (float)d_army->getStat(Army::HP) * 38;
	DrawLine(4, 64, 38, 64, red, 5);
	DrawLine(4, 64, static_cast<unsigned int>(factor), 64, green, 5);
}

bool ArmyInfo::eventMouseButtonDown(const SDL_MouseButtonEvent* event)
{
	debug("eventMouseButtonDown()");
    
    if (d_army == 0)
        return true;

    if (event->button == SDL_BUTTON_RIGHT)
    {
        //pop up an army information dialog

        ArmyDialog dialog(d_army, 0, PG_Rect(150, 100, 540, 440));

        dialog.Show();
        dialog.RunModal();
        dialog.Hide();
        Redraw();

        return true;
    }

    if (!d_pressable)
        return true;

	if (d_pressed) 
	{
		d_pressed = false;
		d_army->setGrouped(false);
	}
	else
	{
		d_pressed = true;
		d_army->setGrouped(true);
	}

	Redraw();
	return true;
}
