//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "FogMap.h"
#include "GameMap.h"
#include <iostream>
#include <sstream>
#include <fstream>
#include "playerlist.h"

using namespace std;

//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<flush;}
#define debug(x)


FogMap::FogMap()
{
    debug("FogMap()");
    d_size = GameMap::getInstance()->getSize();

    d_fogmap = new TYPE[d_size*d_size];

    fill(COVERED);
}

FogMap::FogMap(XML_Helper* helper)
{
    std::string types;
    
    helper->getData(d_size, "size");
    helper->getData(types, "map", false);

    //create the map
    d_fogmap = new TYPE[d_size*d_size];

    for (int y = 0; y < d_size; y++)
    {
        for (int x = 0; x < d_size; x++)
        {
            //due to the circumstances, types is a long stream of
            //numbers, so read it character for character (no \n's or so)
            d_fogmap[y*d_size + x] = static_cast<TYPE>(types[y*d_size + x]);
        }
    }
}


FogMap::~FogMap()
{
    delete[] d_fogmap;
}

bool FogMap::fill(TYPE type)
{
    for (int i = 0; i < d_size*d_size; i++)
        d_fogmap[i] = type;

    return true;
}

bool FogMap::save(XML_Helper* helper) const
{
    bool retval = true;

    std::stringstream types;

    types << std::endl;
    for (int y = 0; y < d_size; y++)
    {
        for (int x = 0; x < d_size; x++)
        {
            types << static_cast<int>(d_fogmap[y*d_size + x]);
        }
        types << std::endl;
    }

    retval &= helper->openTag("fogmap");
    retval &= helper->saveData("size", d_size);
    retval &= helper->saveData("map", types.str());
    retval &= helper->closeTag();

    return retval;
}

FogMap::TYPE FogMap::getFogTile(int x, int y) const
{
    return d_fogmap[y*d_size + x];
}

void FogMap::alterFogRadius(PG_Point pt, int radius, TYPE new_type)
{
    int x = pt.x - radius;
    int y = pt.y - radius;
    for (int i = 0; i < 2*radius; i++)
    {
        for (int j = 0; y < 2*radius; y++)
        {
            if ((x+i) < 0 || (y+j) < 0 || (x+i) >= d_size || (y+j) >= d_size)
                continue;
            d_fogmap[(y+j)*d_size + (x+i)] = new_type;
        }
    }
}

// End of file
