//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef ITEMDIALOG_H
#define ITEMDIALOG_H

#include <pgwindow.h>
#include <pglabel.h>
#include <pgbutton.h>
#include "hero.h"

/** A dialog which allows the player to take and exchange items.
  * 
  * The purpose of the dialog is to have the player exchange the items of his 
  * heroes. Each modification takes place immediately.
  *
  * TODO: - add drag'n drop
  *       - add showing of item abilities
  *       - remove workaround for no icon bug of paragui when upgrading to unstable paragui
  *       - implement colors from game map for move boni
  *       - more item descriptions as more army abilities are implemented
  */

class ItemDialog : public PG_Window
{
    public:
        /** The constructor
          * 
          * Note that the dialog gets the information about the items on the
          * ground from the hero's location.
          * 
          * @param hero     the hero whose items are arranged
          * @param parent   the parent widget (derived from paragui)
          * @param rect     the rectangle where we draw to (derived from paragui)
          */
        ItemDialog(Hero* hero, PG_Widget* parent, PG_Rect rect);
        ~ItemDialog();

        //! Called whenever the player clicks on a button associated with an item
        bool itemClick(PG_Button* btn);

        //! Called whenever the player clicks on a button associated with
        //! scrolling items on the ground.
        bool groundScroll(PG_Button* btn);

        //! Called whenever the player clicks on a button associated with
        //! scrolling items in the backpack.
        bool backScroll(PG_Button* btn);

        //! Called when the player clicks on the OK button
        bool okClicked(PG_Button* btn);

        
    private:
        //! Updates the equipment slots with the current equipment
        void drawEquipment();

        //! Updates the backpack slots
        void drawBackpack();

        //! Updates the ground slots
        void drawGround();
        
        //! Returns the item associated with a certain button (none if button is not occupied).
        Item* getItem(PG_Button* button, bool erase=false);

        //! Returns true if the button is one of the heroes equipment
        bool isEquipment(PG_Button* button);

        //! Inserts an item at the position associated with the button. Returns false on error.
        bool insertItem(Item* item, PG_Button* pos);

        //! This function displays data about an item
        bool displayItem(Item* item);
        
        bool eventKeyDown(const SDL_KeyboardEvent* key);


        Hero* d_hero;
        
        unsigned int d_ground, d_back;
        PG_Point d_pos;
        
        PG_Button* d_b_ground[3];
        PG_Button* d_b_groundnext, *d_b_groundprev;
        PG_Label*  d_l_ground;
        
        PG_Button* d_b_back[3];
        PG_Button* d_b_backnext, *d_b_backprev;
        PG_Label*  d_l_back;
        
        PG_Button* d_b_pic;
        PG_Button* d_b_weapon;
        PG_Button* d_b_shield;
        PG_Button* d_b_armour;
        PG_Button* d_b_accessoire;
        
        PG_Label* d_l_weapon;
        PG_Label* d_l_shield;
        PG_Label* d_l_armour;
        PG_Label* d_l_accessoire;

        PG_Label* d_l_item;
        PG_Label* d_l_bonus[3];

        PG_Button* d_b_ok;

        PG_Button* d_active;

        SDL_Surface* d_tempsurf;
};

#endif //ITEMDIALOG_H
