//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "splash.h"
#include <iostream>
#include <SDL_image.h>
#include "MainWindow.h"
#include "defs.h"
#include "armysetlist.h"
#include "stacklist.h"
#include "playerlist.h"
#include "Configuration.h"
#include "MultiPlayerModeDialog.h"
#include "GraphicsCache.h"
#include "PG_FileDialog.h"
#include "CreateScenario.h"
#include "File.h"
#include "events/RWinGame.h"
#include "events/RLoseGame.h"
#include <pgapplication.h>
#include <pgmessagebox.h>
#include "LangDialog.h"
#include "netggz.h"
#include "w_edit.h"
#include <pthread.h>

#ifdef __WIN32__
  // on mingw32, fd_set lives in winsock.h
  #include <winsock.h>
#else 
  #include <sys/select.h>
#endif

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
//#define debug(x)

Splash::Splash(PG_Application* app, const PG_Rect& rect)
    :PG_ThemeWidget(0, rect), d_mainWindow(0), d_networkcancelled(false)
{
    d_app = app;
    d_background = File::getMiscPicture("splash_screen.jpg", false);
    SetBackground(d_background, 2);
    
	if(!Configuration::s_ggz)
    {
        d_b_campaign = new PG_Button(this,
            PG_Rect(rect.w - 210, rect.h - 250, 180, 30), _("New Campaign"),0);
        d_b_singleplayer_game = new PG_Button(this,
            PG_Rect(rect.w - 210, rect.h - 215, 180, 30), _("New Game"),1);
        d_b_load_game = new PG_Button(this,
            PG_Rect(rect.w - 210, rect.h - 180, 180, 30), _("Load Game"),4);
        d_b_tutorial = new PG_Button(this,
            PG_Rect(rect.w - 210, rect.h - 145, 180, 30), _("Tutorial"),3);
    }
    else
    {
        d_b_campaign = 0;
        d_b_singleplayer_game = 0;
        d_b_load_game = 0;
        d_b_tutorial = 0;
    }
    d_b_multiplayer_game = new PG_Button(this,
            PG_Rect(rect.w - 210, rect.h - 110, 180, 30), _("New Network Game"),2);
    d_b_lang = new PG_Button(this, PG_Rect(rect.w - 210, rect.h - 75, 180, 30),
            _("Language"), 4);
#ifdef __WIN32__
    d_b_lang->SetTransparency(100);
    d_b_lang->EnableReceiver(false);
#endif

    d_b_quit = new PG_Button(this,
            PG_Rect(rect.w - 210, rect.h - 40, 180, 30), _("Quit"),5);


    if(d_b_campaign)
        d_b_campaign->sigClick.connect(slot(*this, &Splash::b_campaignClicked));
    if(d_b_singleplayer_game)
        d_b_singleplayer_game->sigClick.connect(slot(*this, &Splash::b_singleplayerGameClicked));
    d_b_multiplayer_game->sigClick.connect(slot(*this, &Splash::b_multiplayerGameClicked));
    if(d_b_load_game)
        d_b_load_game->sigClick.connect(slot(*this, &Splash::b_loadGameClicked));
    if(d_b_tutorial)
        d_b_tutorial->sigClick.connect(slot(*this, &Splash::b_tutorialClicked));

    d_b_lang->sigClick.connect(slot(*this, &Splash::b_langClicked));
    d_b_quit->sigClick.connect(slot(*this, &Splash::b_quitClicked));

    W_Edit::sigChangeResolution.connect(slot(*this, &Splash::b_resolutionChanged));

    // finally, load the music
#ifdef FL_SOUND
    d_canplay_music=false;
    if (SDL_InitSubSystem(SDL_INIT_AUDIO)==0)
    {
        if(Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, 2, 1024)==0) 
        {
            d_canplay_music=true;
            d_music = File::getMusic("music1.ogg");
            // half volume, to be on the safe side
            Mix_VolumeMusic(Configuration::s_musicvolume);
            if (Configuration::s_musicenable)
                Mix_PlayMusic(d_music, -1);
        }
    }
#endif
}

Splash::~Splash()
{
    SDL_FreeSurface(d_background);
    clearData();
#ifdef FL_SOUND
    if (d_canplay_music) 
    {
        Mix_FreeMusic(d_music);
        Mix_CloseAudio();
        SDL_QuitSubSystem(SDL_INIT_AUDIO);
    }
#endif
    delete d_b_campaign;
    delete d_b_singleplayer_game;
    delete d_b_multiplayer_game;
    delete d_b_load_game;
    delete d_b_tutorial;
    delete d_b_quit;
}


bool Splash::b_resolutionChanged(bool smaller)
{
   
    int w=Configuration::s_width;
    int h=Configuration::s_height;

    debug("SPLASH w=" << w << " h=" << h)
    if (!smaller) 
    {
        d_app->InitScreen(w,h, 16,Configuration::s_flags);
        SizeWidget(w,h);
    }

    if (d_b_campaign) d_b_campaign->MoveWidget(PG_Rect(w - 210, h - 250, 180, 30));
    if (d_b_singleplayer_game) d_b_singleplayer_game->MoveWidget(PG_Rect(w - 210, h - 215, 180, 30));
    if (d_b_load_game) d_b_load_game->MoveWidget(PG_Rect(w - 210, h - 180, 180, 30));
    if (d_b_tutorial) d_b_tutorial->MoveWidget(PG_Rect(w - 210,h - 145, 180, 30));
    d_b_multiplayer_game->MoveWidget(PG_Rect(w - 210,h - 110, 180, 30));
    d_b_lang->MoveWidget(PG_Rect(w - 210, h - 75, 180, 30));
    d_b_quit->MoveWidget(PG_Rect(w - 210, h - 40, 180, 30));

    if (smaller) 
    {
        d_app->InitScreen(w,h, 16,Configuration::s_flags);
        my_height = h-1;
        my_width = w-1;  
        SizeWidget(w,h);
    }
    debug("I arrive here.")
  
    return true;
}

bool Splash::b_quitClicked(PG_Button* btn)
{
	d_app->Quit();
	return true;
}

bool Splash::b_langClicked(PG_Button* btn)
{
    // First, have the user choose the language
    LangDialog dialog(this, PG_Rect(my_width/2 - 150, my_height/2 - 150, 300, 300));
    dialog.Show();
    dialog.RunModal();
    dialog.Hide();

    // Then, rename all buttons
    if (d_b_campaign)
        d_b_campaign->SetText(_("New Campaign"));
    if (d_b_singleplayer_game)
        d_b_singleplayer_game->SetText(_("New Game"));
    if (d_b_load_game)
        d_b_load_game->SetText(_("Load Game"));
    if (d_b_tutorial)
        d_b_tutorial->SetText(_("Tutorial"));
    
    d_b_multiplayer_game->SetText(_("New Network Game"));
    d_b_lang->SetText(_("Language"));
    d_b_quit->SetText(_("Quit"));
    
    return true;
}

bool Splash::b_cancelClicked(PG_Button* btn)
{
	d_network->QuitModal();
	d_networkcancelled = true;
	return true;
}

bool Splash::b_singleplayerGameClicked(PG_Button* btn)
{
	clearData();
	if (newGame());
        return true;

#ifdef FL_SOUND
    if (d_canplay_music && Configuration::s_musicenable)
        Mix_PlayMusic(d_music, -1);
#endif

	return true;
}

bool Splash::newGame(std::string ip, int port)
{
    CreateScenario* creator = new CreateScenario(this);
    GamePreferencesDialog* dialog = 0;
    
    //CreateScenario requires to define a maptype
    creator->setMaptype(CreateScenario::NORMAL);

    //the player has to choose which player to activate etc.
    
    dialog = new GamePreferencesDialog(0, PG_Rect(0, 0,
            PG_Application::GetScreenWidth(),
            PG_Application::GetScreenHeight()));

    dialog->Show();
    dialog->initGUI();
    if(Configuration::s_ggz)
    {
        dialog->restrictPlayers(GGZ::ref()->seats());
    }
    dialog->RunModal();
    dialog->Hide();

    if (dialog->cancelled())
    {
        delete dialog;
        return false;
    }

    if (!dialog->loadedMap())
        dialog->fillData(creator);

    //now create the map and dump the created map under ~savepath/random.map
    std::string randomfilename = File::getSavePath();
    randomfilename += "random.map";
    
    //!dialog means testing game
    if (!dialog || (!dialog->loadedMap()))
    {
        creator->create();
        creator->dump(randomfilename);
    }
    delete creator;

	//if in GGZ mode, wait for other players
    if(Configuration::s_ggz)
    {
        PG_Rect rect = PG_Rect(0, 0, Width(), Height());
        d_network = new PG_ThemeWidget(0, rect);
        SDL_Surface* background = File::getMiscPicture("network_screen.jpg", false);
        d_network->SetBackground(background, 2);

        PG_Button* cancel = new PG_Button(d_network,
                PG_Rect(rect.w - 210, rect.h - 40, 180, 30), _("Cancel"),5);
        cancel->sigClick.connect(slot((*this), &Splash::b_cancelClicked));

        Hide();
        d_network->Show();
		d_network->RunModal();
        d_network->Hide();
        SDL_FreeSurface(background);
        delete d_network;

        if (d_networkcancelled)
        {
            Show();
            delete dialog;
            return false;
        }
    }

    //and finally load the "savegame", set up the main window and start
    bool broken;
    GameScenario* gameScenario = 0;
    
    if (dialog->loadedMap())
        gameScenario = new GameScenario(dialog->getFilename(), broken, true);
    else
        gameScenario = new GameScenario(randomfilename, broken, true);

    if (broken)
    {
        std::cerr <<_("random.map was broken when re-reading. Exiting...\n");
        exit(-1);
    }

    //set up misc stuff
    RWinGame::swinning.connect(SigC::slot((*this), &Splash::gameFinished));
    RLoseGame::slosing.connect(SigC::slot((*this), &Splash::gameFinished));

    Hide();
    
    if (dialog)
        delete dialog;
    
    d_mainWindow = new MainWindow(gameScenario,
                                    PG_Rect(0, 0, Width(), Height()));
    d_mainWindow->squitting.connect(SigC::slot((*this),
                                    &Splash::gameFinished));
    d_mainWindow->Show();
    d_mainWindow->startGame();

    return true;
}

bool Splash::b_multiplayerGameClicked(PG_Button* btn)
{
    clearData();

    if(Configuration::s_ggz)
    {
		GGZ::ref()->init();
		bool success = GGZ::ref()->connect();
		if(!success)
    	{
        	std::cerr << _("GGZ initialization failed. Exiting.\n") << std::endl;
        	exit(-1);
			return false;
    	}
        std::cout << "GGZ: playing with " << GGZ::ref()->seats() << " players." << std::endl;

        //d_app->EnableAppIdleCalls(true);
        //d_app->sigAppIdle.connect(slot((*this), &Splash::networkInput));

        pthread_t id;
        pthread_create(&id, NULL, &Splash::networkThread, (void*)this);

        newGame();
        return false;
    }
	else
    {
        PG_MessageBox mb(GetParent(), PG_Rect(200, 200, 200, 150),
                _("Play on GGZ"),
                _("Please launch Freelords from a GGZ Gaming Zone client."),
                PG_Rect(60, 100, 80, 30), _("OK"));
        mb.Show();
        mb.RunModal();
        mb.Hide();
    }

    /*MultiPlayerModeDialog dialog(this, PG_Rect(20, 20, 350, 230));
    dialog.Show();
    dialog.RunModal();
    dialog.Hide();

    if (dialog.getResult())
    {
        if (dialog.getMode())
        {
            newGame(GameScenario::MULTI_PLAYER_HOST, "", dialog.getPort());
        }
        else
        {  
            newGame(GameScenario::MULTI_PLAYER_CLIENT, dialog.getIP(), dialog.getPort());
        }
    }*/

    return true;
}

bool Splash::b_loadGameClicked(PG_Button* btn)
{
    PG_FileDialog opendialog(0, PG_FileDialog::PG_OPEN, true,
            Configuration::s_savePath, "*.sav", _("FreeLords savegames"));
    opendialog.Show();
    opendialog.RunModal();
    std::string savegame = opendialog.getSelectedFile();

    //no file selected
    if (savegame.empty())
    {
        return true;
    }
    
    clearData();

    bool broken;
    GameScenario* gameScenario = new GameScenario(savegame, broken, true);

    if (broken)
    {
        std::cerr <<savegame <<_(": Could not load savegame. Exiting.\n");
        exit(-1);
    }

    RWinGame::swinning.connect(SigC::slot((*this), &Splash::gameFinished));
    RLoseGame::slosing.connect(SigC::slot((*this), &Splash::gameFinished));

    Hide();
        
    d_mainWindow = new MainWindow(gameScenario, PG_Rect(0, 0, Width(), Height()));
    d_mainWindow->squitting.connect(SigC::slot((*this), &Splash::gameFinished));
    d_mainWindow->Show();
    d_mainWindow->loadGame(savegame);

    return true;
}

bool Splash::b_tutorialClicked(PG_Button* btn)
{
    debug(_("TUTORIAL Clicked!"));

    clearData();

    std::string savegame = Configuration::s_dataPath+string("/map/tutorial.map");

    bool broken;
    GameScenario* gameScenario = new GameScenario(savegame, broken, true);

    if (broken)
    {
        std::cerr <<savegame <<_(": Could not load savegame. Exiting.\n");
        exit(-1);
    }

    RWinGame::swinning.connect(SigC::slot((*this), &Splash::gameFinished));
    RLoseGame::slosing.connect(SigC::slot((*this), &Splash::gameFinished));

    Hide();
        
    d_mainWindow = new MainWindow(gameScenario, PG_Rect(0, 0, Width(), Height()));
    d_mainWindow->squitting.connect(SigC::slot((*this), &Splash::gameFinished));
    d_mainWindow->Show();
    d_mainWindow->startGame();
    return true;
}

bool Splash::b_campaignClicked(PG_Button* btn)
{
    clearData();
    PG_MessageBox mb(GetParent(), PG_Rect(200, 200, 200, 150),
            _("Not implemented"),
            _("Campaigns are not implemented yet."),
            PG_Rect(60, 100, 80, 30), _("OK"));
    mb.Show();
    mb.RunModal();
    mb.Hide();
    return true;
}

void Splash::clearData()
{
    if (d_mainWindow)
    {
        delete d_mainWindow;
        d_mainWindow = 0;
    }

    // some of the references here use player pointers which are obsolete by now,
    // beside it assumes some specific terrain set and such
    GraphicsCache::deleteInstance();

    // stop the music from playing
#ifdef FL_SOUND
    if (d_canplay_music) 
    {
        Mix_HaltMusic();
    }
#endif
}

void Splash::gameFinished(Uint32 status)
{
    debug("gameFinished()");

    //the memory for gameScenario and mainWindow is freed as soon as a
    //new game is started

    d_mainWindow->stopGame();
    d_mainWindow->Hide();

    Show();

    // restart music
#ifdef FL_SOUND
    if (d_canplay_music && Configuration::s_musicenable) 
    {
        Mix_PlayMusic(d_music, -1);
    }
#endif
}

void *Splash::networkThread(void *arg)
{
    Splash *splash = (Splash*)arg;

    int fd = GGZ::ref()->fd();
    fd_set set;
    FD_ZERO(&set);     
    FD_SET(fd, &set);
    int ret;
    struct timeval tv;

    while(1)
    {
        tv.tv_sec = 1;
	tv.tv_usec = 0;

        ret = select(fd + 1, &set, NULL, NULL, &tv);
        printf("GGZ (thread) ## check (seats: %i)\n", GGZ::ref()->seats());

        if(ret > 0)
        {
            printf("GGZ (thread) ## got data\n");
            splash->networkInput();
        }
        else if(ret == -1)
        {
            printf("GGZ (thread) ## select error\n");
            GGZ::ref()->deinit();
            return NULL;
        }
    }

    return NULL;
}

bool Splash::networkInput()
{
	//debug("check network!");

	if(GGZ::ref()->data())
		networkData();

	return true;
}

void Splash::networkData()
{
#ifdef WITH_GGZ
	int fd, ret, op;

	fd = GGZ::ref()->fd();

	ret = ggz_read_int(fd, &op);
	printf("GGZ ## received opcode: %i (%i)\n", op, ret);
	if(ret < 0)
	{
		GGZ::ref()->deinit();
		return;
	}

	if(op == 42)
	{
		debug("opcode 42: initialisation!");
	}
	else
	{
		debug("unknown opcode!");
	}
#endif
}

bool Splash::eventKeyDown(const SDL_KeyboardEvent* key)
{	
    // Check , which key was pressed
    switch (key->keysym.sym)
    {		
        case SDLK_f:		        
	  //d_w_edit->lockScreen();
             if(Configuration::s_fullScreen)
	     {
                 Configuration::s_flags&= ~SDL_FULLSCREEN;	
                 PG_Application::GetApp()->InitScreen(Configuration::s_width, Configuration::s_height, 16,Configuration::s_flags);
                 Configuration::s_fullScreen=false; 
	     }
             else 
	     {
                 Configuration::s_flags|=SDL_FULLSCREEN;	
                 PG_Application::GetApp()->InitScreen(Configuration::s_width, Configuration::s_height, 16,Configuration::s_flags);
                 Configuration::s_fullScreen=true; 
	     }
             break;
	 default:
	     break;	
    }	
    return true;
}
